@@ -13,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA 02110-1301  USA
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA
  */
 
 #include <config.h>
@@ -59,10 +59,13 @@ gucharmap_chapters_view_init (GucharmapChaptersView *view)
   priv = view->priv = G_TYPE_INSTANCE_GET_PRIVATE (view, GUCHARMAP_TYPE_CHAPTERS_VIEW, GucharmapChaptersViewPrivate);
 
   cell = gtk_cell_renderer_text_new ();
+  g_object_set (cell, 
+                "ellipsize", PANGO_ELLIPSIZE_END,
+                "ellipsize-set", TRUE,
+                NULL);
   column = priv->column = gtk_tree_view_column_new ();
   gtk_tree_view_column_pack_start (column, cell, FALSE);
   gtk_tree_view_column_add_attribute (column, cell, "text", GUCHARMAP_CHAPTERS_MODEL_COLUMN_LABEL);
-  gtk_tree_view_column_set_sort_column_id (column, GUCHARMAP_CHAPTERS_MODEL_COLUMN_LABEL);
   gtk_tree_view_append_column (tree_view, column);
 
   selection = gtk_tree_view_get_selection (tree_view);
@@ -121,8 +124,9 @@ gucharmap_chapters_view_set_model (GucharmapChaptersView *view,
     return;
 
   gtk_tree_view_column_set_title (priv->column, gucharmap_chapters_model_get_title (model));
+  gtk_tree_view_column_set_sort_column_id (priv->column, model->priv->sort_column);
   gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (model),
-                                        GUCHARMAP_CHAPTERS_MODEL_COLUMN_LABEL,
+                                        model->priv->sort_column,
                                         GTK_SORT_ASCENDING);
 
   /* Need to re-set this here since it's set to -1 when the tree view model changes! */
